#ifndef _FIVEWIN_CH
#define _FIVEWIN_CH

#include "Window.ch"
#include "Brush.ch"
#include "Dbf.ch"
#include "DDE.ch"
#include "Dialog.ch"
#include "DLL.ch"
#include "Font.ch"
#include "Icon.ch"
#include "Ini.ch"
#include "Menu.ch"
#include "Objects.ch"
#include "Print.ch"
#include "Timer.ch"
#include "Video.ch"
#include "WinApi.ch"
#include "Meter.ch"

//----------------------------------------------------------------------------//
// ACCESSING / SETTING Variables

#xtranslate bSETGET(<uVar>) => ;
            { | u | If( PCount() == 0, <uVar>, <uVar> := u ) }

//----------------------------------------------------------------------------//

#xcommand SET HELPFILE TO <cFile>    => SetHelpFile( <cFile> )
#xcommand SET HELP TOPIC TO <cTopic> => HelpSetTopic( <cTopic> )

//----------------------------------------------------------------------------//
// Microsoft 3D Look

#xcommand SET <look_3d: 3DLOOK, LOOK3D, LOOK 3D, 3D LOOK> <on:ON,OFF,&> => ;
          Set3DLook( Upper(<(on)>) == "ON" )

//----------------------------------------------------------------------------//
// Resources

#xcommand SET RESOURCES TO <cName> => SetResources( <cName> )

#xcommand SET RESOURCES TO => FreeResources()

//----------------------------------------------------------------------------//
// Loading strings/or other Types from Resources

#xcommand REDEFINE <uVar> ;
             [ AS <type: CHARACTER, NUMERIC, LOGICAL, DATE> ] ;
             [ <resource: RESOURCE, RESNAME, NAME> <nIdRes> ] ;
       => ;
          <uVar> := LoadValue( <nIdRes>, [Upper(<(type)>)], <uVar> )

//----------------------------------------------------------------------------//
// Colores ms frecuentes

#define CLR_BLACK            0
#define CLR_GRAY       8421504               // RGB( 128, 128, 128 )
#define CLR_LIGHTGRAY 12632256               // RGB( 192, 192, 192 )
#define CLR_WHITE     16777215               // RGB( 255, 255, 255 )

//---------------------------------------------------------------------------//
// MESSAGE BAR

#xcommand SET MESSAGE [ OF <oWnd> ] [ TO <cMsg> ] => ;
          <oWnd>:oMsgBar := TMsgBar():New( <oWnd>, <cMsg> )
          
#xcommand DEFINE MESSAGE BAR OF <oWnd> ;
             [ PROMPT <cMsg> ] ;
      => ;
         <oWnd>:oMsgBar := TMsgBar():New( <oWnd>, <cMsg> )

#xcommand SET MENU OF <oWnd> TO <oMenu> => <oWnd>:SetMenu( <oMenu> )

//----------------------------------------------------------------------------//
// PUSHBUTTON

#xcommand @ <nRow>, <nCol> BUTTON [ <oBtn> PROMPT ] <cCaption> ;
             [ SIZE <nWidth>, <nHeight> ] ;
             [ ACTION <uAction> ] ;
             [ <default: DEFAULT> ] ;
             [ <of:OF, WINDOW, DIALOG> <oWnd> ] ;
             [ <help:HELP, HELPID, HELP ID> <nHelpId> ] ;
             [ FONT <oFont> ] ;
      => ;
         [ <oBtn> := ] TButton():New( <nRow>, <nCol>, <cCaption>, <oWnd>,;
            <{uAction}>, <nWidth>, <nHeight>, <nHelpId>, <oFont>, <.default.> )

#xcommand REDEFINE BUTTON [ <oBtn> ] ;
             [ ID <nId> <of:OF, WINDOW, DIALOG> <oDlg> ] ;
             [ ACTION <uAction> ] ;
             [ <help:HELP, HELPID, HELP ID> <nHelpId> ] ;
       => ;
          [ <oBtn> := ] TButton():ReDefine( <nId>, <{uAction}>, <oDlg>,;
             <nHelpId> )

//----------------------------------------------------------------------------//
// CHECKBOX

#xcommand REDEFINE CHECKBOX [ <oCbx> VAR ] <lVar> ;
             [ ID <nId> ] ;
             [ <of:OF, WINDOW, DIALOG> <oWnd> ] ;
             [ <help:HELPID, HELP ID> <nHelpId> ] ;
             [ ON CLICK <uClick> ];
             [ VALID   <ValidFunc> ]       ;
       => ;
          [ <oCbx> := ] TCheckBox():ReDefine( <nId>, bSETGET(<lVar>),;
             <oWnd>, <nHelpId>, [<{uClick}>], <{ValidFunc}>  )

#xcommand @ <nRow>, <nCol> CHECKBOX [ <oCbx> VAR ] <lVar> ;
             [ PROMPT <cCaption> ] ;
             [ <of:OF, WINDOW, DIALOG> <oWnd> ] ;
             [ SIZE <nWidth>, <nHeight> ] ;
             [ <help:HELPID, HELP ID> <nHelpId> ] ; 
             [ FONT <oFont> ] ;
             [ ON CLICK <uClick> ];
             [ VALID   <ValidFunc> ]       ;
      => ;
         [ <oCbx> := ] TCheckBox():New( <nRow>, <nCol>, <cCaption>,;
             [bSETGET(<lVar>)], <oWnd>, <nWidth>, <nHeight>, <nHelpId>,;
             [<{uClick}>], <oFont>, <{ValidFunc}> )

//----------------------------------------------------------------------------//
// COMBOBOX

#xcommand @ <nRow>, <nCol> COMBOBOX [ <oCbx> VAR ] <cVar> ;
             [ ITEMS <aItems> ] ;
             [ SIZE <nWidth>, <nHeight> ] ;
             [ <dlg:OF,WINDOW,DIALOG> <oWnd> ] ;
             [ <help:HELPID, HELP ID> <nHelpId> ] ;
             [ ON CHANGE <uChange> ] ;
             [ VALID   <ValidFunc> ] ;
       => ;
          [ <oCbx> := ] TComboBox():New( <nRow>, <nCol>, bSETGET(<cVar>),;
             <aItems>, <nWidth>, <nHeight>, <oWnd>, <nHelpId>,;
             [{|Self|<uChange>}], <{ValidFunc}> )

#xcommand REDEFINE COMBOBOX [ <oCbx> VAR ] <cVar> ;
             [ ITEMS <aItems> ] ;
             [ ID <nId> ] ;
             [ <dlg:OF,WINDOW,DIALOG> <oWnd> ] ;
             [ <help:HELPID, HELP ID> <nHelpId> ] ;
             [ ON CHANGE <uChange> ] ;
             [ VALID   <ValidFunc> ]       ;
       => ;
          [ <oCbx> := ] TComboBox():ReDefine( <nId>, bSETGET(<cVar>),;
             <aItems>, <oWnd>, <nHelpId>, <{ValidFunc}>, [{|Self|<uChange>}] )

//----------------------------------------------------------------------------//
// LISTBOX

#xcommand REDEFINE LISTBOX [ <oLbx> VAR ] <cVar> ;
             [ ITEMS <aItems> ]  ;
             [ FILES <cFileSpec> ] ;
             [ ID <nId> ] ;
             [ ON CHANGE <uChange> ] ;
             [ <of: OF, WINDOW, DIALOG > <oWnd> ] ;
             [ <help:HELPID, HELP ID> <nHelpId> ] ;
             [ <bitmaps: BITMAPS > [ SIZE <nSize> ] ] ;
             [ VALID   <ValidFunc> ]       ;
       => ;
          [ <oLbx> := ] TListBox():ReDefine( <nId>, bSETGET(<cVar>), <aItems>,;
             <{uChange}>, <oWnd>, <nHelpId>, <.bitmaps.>, <nSize>, ;
             <{ValidFunc}>, <cFileSpec> )

#xcommand @ <nRow>, <nCol> LISTBOX [ <oLbx> VAR ] <cVar> ;
             [ ITEMS <aList>  ] ;
             [ SIZE <nWidth>, <nHeight> ] ;
             [ ON CHANGE <uChange> ] ;
             [ <of: OF, WINDOW, DIALOG > <oWnd> ] ;
             [ VALID   <ValidFunc> ]       ;
       => ;
          [ <oLbx> := ] TListBox():New( <nRow>, <nCol>, bSETGET(<cVar>),;
             <aList>, <nWidth>, <nHeight>, <{uChange}>, <oWnd>, <{ValidFunc}> )

//----------------------------------------------------------------------------//
// LISTBOX - BROWSE

#command REDEFINE LISTBOX [ <oLbx> ] FIELDS [<Flds,...>] ;
             [ ID <nId> ] ;
             [ <dlg:OF,DIALOG> <oDlg> ] ;
             [ FIELDSIZES <aColSizes,...> ] ;
             [ <head:HEAD,HEADER,HEADERS> <aHeaders,...> ] ;
       => ;
              [ <oLbx> := ] TWBrowse():ReDefine( <nId>, ;
                                       [\{|| \{ <Flds> \} \}], <oDlg>,;
                                        [ \{<aHeaders>\}], [\{<aColSizes>\}] )

#command @ <nRow>, <nCol> LISTBOX [ <oBrw> ] FIELDS [<Flds,...>] ;
                           [ FIELDSIZES <aColSizes,...> ] ;
                           [ <head:HEAD,HEADER,HEADERS> <aHeaders,...> ] ;
                           [ ID <nId> ] ;
                           [ SIZE <nWidth>, <nHeigth> ] ;
                           [ <dlg:OF,DIALOG> <oDlg> ] ;
                                                          => ;
          [ <oBrw> := ] TWBrowse():New( <nRow>, <nCol>, <nWidth>, <nHeigth>,;
                                       [\{|| \{<Flds> \} \}], ;
                                       [ \{<aHeaders>\}], [\{<aColSizes>\}], ;
                                       <nId>, <oDlg> )

//----------------------------------------------------------------------------//
// RADIOBUTTONS

#xcommand @ <nRow>, <nCol> RADIO [ <oRadMenu> VAR ] <nVar> ;
             [ PROMPT <cItems,...> ] ;
             [ <of: OF, WINDOW, DIALOG> <oWnd> ] ;
             [ <help:HELPID, HELP ID> <nHelpId> ] ; 
       => ;
          [ <oRadMenu> := ] TRadMenu():New( <nRow>, <nCol>, {<cItems>},;
                                   [bSETGET(<nVar>)], <oWnd>, <nHelpId> )

#xcommand REDEFINE RADIO [ <oRadMenu> VAR ] <nVar> ;
             [ ID <nId,...> ] ;
             [ <of: OF, WINDOW, DIALOG> <oWnd> ] ;
             [ <help:HELPID, HELP ID> <nHelpId> ] ;
       => ;
          [ <oRadMenu> := ] TRadMenu():Redefine( [bSETGET(<nVar>)],;
             <oWnd>, <nHelpId>, \{ <nId> \} )

//----------------------------------------------------------------------------//
// BITMAP

#xcommand @ <nRow>, <nCol> BITMAP [ <oBmp> ] ;
             [ <resource: NAME, RESNAME, RESOURCE> <cResName> ] ;
             [ <file: FILE, FILENAME, DISK> <cBmpFile> ] ;
             [ <NoBorder:NOBORDER, NO BORDER> ] ;
             [ SIZE <nWidth>, <nHeight> ] ;
             [ <of: OF, WINDOW, DIALOG> <oWnd> ] ;
       => ;
          [ <oBmp> := ] TBitmap():New( <nRow>, <nCol>, <nWidth>, <nHeight>,;
             <cResName>, <cBmpFile>, <.NoBorder.>, <oWnd> )

#xcommand REDEFINE BITMAP [ <oBmp> ] ;
             [ ID <nId> ] ;
             [ <of: OF, WINDOW, DIALOG> <oWnd> ] ;
             [ <resource: NAME, RESNAME, RESOURCE> <cResName> ] ;
             [ <file: FILE, FILENAME, DISK> <cBmpFile> ] ;
       => ;      
          [ <oBmp> := ] TBitmap():ReDefine( <nId>, <cResName>, <cBmpFile>,;
             <oWnd> )

//----------------------------------------------------------------------------//
// SAY

#xcommand REDEFINE SAY [ <oSay> ] ;
             [ PROMPT <cText> ] ;
             [ PICTURE <cPict> ] ;
             [ ID <nId> ] ;
             [ <dlg:OF,WINDOW,DIALOG> <oWnd> ] ;
             [ <lCenter: CENTERED > ]  ;
       => ;
          [ <oSay> := ] TSay():ReDefine( <nId>, <cText>, <oWnd>, ;
                                         <cPict>, <.lCenter.> )

#xcommand @ <nRow>, <nCol> SAY [ <oSay> PROMPT ]  <cText> ;
             [ PICTURE <cPict> ] ;
             [ COLOR <cColor>] ;
             [ <dlg:OF,WINDOW,DIALOG> <oWnd> ] ;
             [ FONT <oFont> ]  ;
             [ <lCenter: CENTERED > ]  ;
             [ <lBorder: BORDER> ]  ;
             [ <lPixel: PIXEL, PIXELS> ] ;
      => ;
          [ <oSay> := ] TSay():New( <nRow>, <nCol>, <cText>,;
             [<oWnd>], [<cPict>], <oFont>, <.lCenter.>, <.lBorder.>,;
             <lPixel> )

//----------------------------------------------------------------------------//
// GET

#xcommand REDEFINE GET [ <oGet> VAR ] <uVar> ;
             [ <memo: MULTILINE, MEMO, TEXT> ] ;
             [ ID <nId> ] ;
             [ <dlg: OF, WINDOW, DIALOG> <oDlg> ] ;
             [ <help:HELPID, HELP ID> <nHelpId> ] ;
       => ;
          [ <oGet> := ] TMultiGet():ReDefine( <nId>, bSETGET(<uVar>),;
             <oDlg>, <nHelpId> )

#xcommand REDEFINE GET [ <oGet> VAR ] <uVar> ;
             [ ID <nId> ] ;
             [ <dlg: OF, WINDOW, DIALOG> <oDlg> ] ;
             [ <help:HELPID, HELP ID> <nHelpId> ] ;
             [ VALID   <ValidFunc> ]       ;
             [ PICTURE <cPict> ] ;
       => ;
          [ <oGet> := ] TGet():ReDefine( <nId>, bSETGET(<uVar>), <oDlg>,;
             <nHelpId>, <cPict>, <{ValidFunc}> )

#command @ <nRow>, <nCol> GET [ <oGet> VAR ] <uVar> ;
                          [ <dlg: OF, WINDOW, DIALOG> <oWnd> ] ;
                          [ <memo: MULTILINE, MEMO, TEXT> ] ;
                          [ COLOR   <cColor> ] ;
                          [ SIZE <nWidth>, <nHeight> ] ;
       => ;
          [ <oGet> := ] TMultiGet():New( <nRow>, <nCol>, bSETGET(<uVar>),;
                            [<oWnd>], <nWidth>, <nHeight> )

#command @ <nRow>, <nCol> GET [ <oGet> VAR ] <uVar>     ;
                          [ <dlg: OF, WINDOW, DIALOG> <oWnd> ] ;
                          [ PICTURE <cPict> ]           ;
                          [ VALID   <ValidFunc> ]       ;
                          [ WHEN    <WhenFunc> ]        ;
                          [ COLOR   <cColor> ]          ;
                          [ SIZE <nWidth>, <nHeight> ]  ;
       => ;
          [ <oGet> := ] TGet():New( <nRow>, <nCol>, bSETGET(<uVar>),;
                      [<oWnd>], <nWidth>, <nHeight>, <cPict>, <{ValidFunc}> )

//----------------------------------------------------------------------------//
// SCROLLBAR

#xcommand @ <nRow>, <nCol> SCROLLBAR [ <oSbr> ] ;
             [ <h: HORIZONTAL> ] ;
             [ <v: VERTICAL> ] ;
             [ RANGE <nMin>, <nMax> ] ;
             [ PAGESTEP <nPgStep> ] ;
             [ SIZE <nWidth>, <nHeight> ] ;
             [ <up:UP, ON UP> <uUpAction> ] ;
             [ <dn:DOWN, ON DOWN> <uDownAction> ] ;
             [ <pgup:PAGEUP, ON PAGEUP> <uPgUpAction> ] ;
             [ <pgdn:PAGEDOWN, ON PAGEDOWN> <uPgDownAction> ] ;
             [ OF <oWnd> ] ;
       => ;
          [ <oSbr> := ] TScrollBar():New( <nRow>, <nCol>, <nMin>, <nMax>, <nPgStep>,;
             (.not.<.h.>) [.or. <.v.> ], <oWnd>, <nWidth>, <nHeight> ,;
             [<{uUpAction}>], [<{uDownAction}>], [<{uPgUpAction}>], ;
             [<{uPgDownAction}>] )

#xcommand REDEFINE SCROLLBAR [ <oSbr> ] ;
             [ ID <nID>  ] ;
             [ RANGE <nMin>, <nMax> ] ;
             [ PAGESTEP <nPgStep> ] ;
             [ <up:UP, ON UP> <uUpAction> ] ;
             [ <dn:DOWN, ON DOWN> <uDownAction> ] ;
             [ <pgup:PAGEUP, ON PAGEUP> <uPgUpAction> ] ;
             [ <pgdn:PAGEDOWN, ON PAGEDOWN> <uPgDownAction> ] ;
             [ OF <oDlg> ] ;
       => ;
             [ <oSbr> := ] TScrollBar():Redefine( <nID>, <nMin>, <nMax>, <nPgStep>,;
             <oDlg>, [<{uUpAction}>], [<{uDownAction}>], [<{uPgUpAction}>], ;
             [<{uPgDownAction}>] )

//----------------------------------------------------------------------------//
// BOX - GROUPS

#xcommand @ <nTop>, <nLeft> [ GROUP <oGroup> ] TO <nBottom>, <nRight > ;
             [ <label:LABEL,PROMPT> <cLabel> ] ;
             [ OF <oWnd> ] ;
       => ;
          [ <oGroup> := ] TGroup():New( <nTop>, <nLeft>, <nBottom>, <nRight>,;
                                        <cLabel>, <oWnd> )

#xcommand REDEFINE GROUP [ <oGroup> ] ;
             [ <label:LABEL,PROMPT> <cLabel> ] ;
             [ ID <nId> ] ;
             [ <of: OF, WINDOW, DIALOG> <oWnd> ] ;
       => ;
          [ <oGroup> := ] TGroup():ReDefine( <nId>, <cLabel>, <oWnd> )

//----------------------------------------------------------------------------//
// CLIPBOARD

#xcommand DEFINE CLIPBOARD <oClp> ;
             [ FORMAT <format:TEXT,OEMTEXT,BITMAP,DIF> ] ;
             [ OF <oWnd> ] ;
       => ;
          <oClp> := TClipBoard():New( [Upper(<(format)>)], <oWnd> )

#xcommand ACTIVATE CLIPBOARD <oClp>     => <oClp>:Open()

#xcommand SET TEXT OF CLIPBOARD <oClp> ;
             TO <cText> ;
       => ;
          <oClp>:SetText( <cText> )

#xcommand DEACTIVATE CLIPBOARD <oClp>   => <oClp>:Close()

#xcommand RELEASE CLIPBOARD <oClp>      => <oClp>:Close(); <oClp> := nil

//----------------------------------------------------------------------------//
// Visual Basic VBX Controls Support

#xcommand REDEFINE VBX [<oControl>] ;
             [ ID <nId> ] ;
             [ OF <oDlg> ] ;
       => ;
          [ <oControl> := ] TVbControl():ReDefine( <nId>, <oDlg> )

//----------------------------------------------------------------------------//
// Comandos y funciones no compatibles

#xcommand CLS =>

#xcommand CLEAR SCREEN =>

#xcommand READ =>

#xcommand InKey( [<nTime>] ) => ;
   MsgAlert( OemToAnsi( "Funcin no disponible en FiveWin" ) )

#xcommand KEYBOARD <cChars> =>

#xcommand __KeyBoard( <cChars> ) => ;
   MsgAlert( OemToAnsi( "Funcin no disponible en FiveWin" ) )

#xcommand SET KEY <nKey> TO <*func*> =>

#xcommand SAVE SCREEN [ TO <u> ] =>

#xcommand RESTORE SCREEN [ FROM <u> ] =>

#xcommand SaveScreen( <*u*> ) => ;
   MsgAlert( OemToAnsi( "Funcin no disponible en FiveWin" ) )

#xcommand RestScreen( <*u*> ) => ;
   MsgAlert( OemToAnsi( "Funcin no disponible en FiveWin" ) )

#xcommand @ <nRow>, <nCol> PROMPT <*u*> =>

#xcommand MENU TO <u> =>

#command ? [ <list,...> ] => WQout( [ \{ <list> \} ] )

#command ?? [ <list,...> ] => WQout( [ \{ <list> \} ] )

#endif
