#include "FiveWin.ch"

#define CB_ADDSTRING     ( WM_USER +  3 )
#define CB_DELETESTRING  ( WM_USER +  4 )
#define CB_GETCURSEL     ( WM_USER +  7 )
#define CB_INSERTSTRING  ( WM_USER + 10 )
#define CB_SETCURSEL     ( WM_USER + 14 )
#define CB_ERR                       -1

//----------------------------------------------------------------------------//

CLASS TComboBox FROM TControl

   DATA   aItems
   DATA   bChange
   DATA   nAt

   METHOD New( nRow, nCol, bSetGet, aItems, nWidth, nHeight, oWnd, nHelpId,;
               bChange, bValid ) CONSTRUCTOR
   METHOD ReDefine( nId, bSetGet, aItems, oWnd, nHelpId, bValid, ;
               bChange ) CONSTRUCTOR
   METHOD cToChar() BLOCK { | Self | Super:cToChar( "COMBOBOX" ) }
   METHOD Init( hDlg )
   METHOD Change()

   METHOD Refresh() BLOCK ;
          { | Self | ::Set( Eval( ::bSetGet ) ), Super:Refresh() }

   METHOD Select( nItem ) BLOCK ;
                    { | Self, nItem | ::SendMsg( CB_SETCURSEL, nItem, 0 ) }
   METHOD Set( cNewItem )
   MESSAGE SetFocus METHOD _SetFocus( lOnOff )

   METHOD Paint() BLOCK { | Self | Paint3D( ::hWnd ) }

   METHOD Add( cItem, nAt )
   METHOD Modify( cItem, nAt )
   METHOD Insert( cItem, nAt )
   METHOD Del( nAt )

ENDCLASS

//----------------------------------------------------------------------------//

METHOD New( nRow, nCol, bSetGet, aItems, nWidth, nHeight, oWnd, nHelpId,;
            bChange, bValid ) CLASS TComboBox

   DEFAULT aItems := {}, nWidth := 40, nHeight := 60

   ::cCaption = ""
   ::nTop     = nRow * 14
   ::nLeft    = nCol *  8
   ::nBottom  = ::nTop  + nHeight - 1
   ::nRight   = ::nLeft + nWidth  - 1
   ::nAt      = 1
   ::aItems   = aItems
   ::bChange  = bChange
   ::bSetGet  = bSetGet
   ::oWnd     = oWnd
   ::nStyle   = nOR( LBS_NOTIFY, WS_TABSTOP, CBS_DROPDOWNLIST,;
                     LBS_DISABLENOSCROLL, WS_CHILD, WS_VISIBLE, WS_BORDER,;
                     WS_VSCROLL )
   ::nId      = ::GetNewId()
   ::nHelpId  = nHelpId
   ::bValid   = bValid

   if oWnd:ClassName() != "TDIALOG"
      ::Create( "COMBOBOX" )
   else
      oWnd:DefControl( Self )
   endif

return

//----------------------------------------------------------------------------//

METHOD ReDefine( nId, bSetGet, aItems, oWnd, nHelpId, bValid, ;
                 bChange ) CLASS TComboBox
   DEFAULT aItems := {}

   ::nId     = nId
   ::hWnd    = 0
   ::aItems  = aItems
   ::bChange = bChange
   ::bSetGet = bSetGet
   ::oWnd    = oWnd
   ::nHelpId = nHelpId
   ::bValid  = bValid
   ::nAt     = 1

   oWnd:DefControl( Self )

return

//----------------------------------------------------------------------------//

METHOD Init( hDlg ) CLASS TComboBox

   local cStart := Eval( ::bSetGet )

   Super:Init( hDlg )
   AEval( ::aItems, { | cItem | ::Add( cItem ) } )
   ::nAt = AScan( ::aItems, { | cItem | cItem == cStart } )
   ::Select( ::nAt - If( ::nAt > 0, 1, 0 ) )

return

//----------------------------------------------------------------------------//

METHOD Change() CLASS TComboBox

   ::nAt = ::SendMsg( CB_GETCURSEL ) + 1
   Eval( ::bSetGet, ::aItems[ ::nAt ] )

   if ::bChange != nil
      Eval( ::bChange, Self )
   endif

return

//----------------------------------------------------------------------------//

METHOD Set( cNewItem ) CLASS TComboBox

   local nAt := AScan( ::aItems,;
                       { | cItem | RTrim( cItem ) == RTrim( cNewItem ) } )

   if nAt != 0
      ::Select( nAt - 1 )
   endif

return

//----------------------------------------------------------------------------//

METHOD _SetFocus( lOnOff ) CLASS TComboBox

   local nAt

   if ! lOnOff
      nAt = ::SendMsg( CB_GETCURSEL )
      if nAt != CB_ERR
         ::nAt = nAt
         Eval( ::bSetGet, ::aItems[ nAt + 1 ] )
      endif
   endif

return

//----------------------------------------------------------------------------//

METHOD Add( cItem, nAt ) CLASS TComboBox

   DEFAULT nAt := 0

   if nAt == 0
      AAdd( ::aItems, cItem )
   else
      ASize( ::aItems, Len( ::aItems ) + 1 )
      AIns( ::aItems, nAt )
      ::aItems[ nAt ] = cItem
   endif

   ::SendMsg( CB_ADDSTRING, nAt, cItem )

return

//----------------------------------------------------------------------------//

METHOD Modify( cItem, nAt ) CLASS TComboBox

   DEFAULT nAt := 0

   if nAt != 0
      ::aItems[ nAt ] = cItem
      ::SendMsg( CB_DELETESTRING, nAt - 1 )
      ::SendMsg( CB_INSERTSTRING, nAt - 1, cItem )
   endif

return

//----------------------------------------------------------------------------//

METHOD Insert( cItem, nAt ) CLASS TComboBox

   DEFAULT nAt := 0

   if nAt != 0
      ASize( ::aItems, Len( ::aItems ) + 1 )
      AIns( ::aItems, nAt )
      ::aItems[ nAt ] = cItem
      ::SendMsg( CB_INSERTSTRING, nAt - 1, cItem )
   endif

return

//----------------------------------------------------------------------------//

METHOD Del( nAt ) CLASS TComboBox

   DEFAULT nAt := 0

   if nAt != 0
      ADel( ::aItems, nAt )
      ASize( ::aItems, Len( ::aItems ) - 1 )
      ::SendMsg( CB_DELETESTRING, nAt - 1 )
   endif

return

//----------------------------------------------------------------------------//
